/*
    This file is part of the 'yarec' package.
    Copyright (C) 1998  Ralf W. Stephan <stephan@tmt.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "uiHandler.h"

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "util/util.h"
#include "tasks/tasks.h"

void CLIUI_msg (const char*);
static int get_key();
static int stopped_f = 0;

static void task_play(), task_record();

void CLIUI_init()
{
  stopped_f = 0;
}

int CLIUI_is_commandline()
{
  return 1;
}

void CLIUI_print_to_console (const char* s1, const char* s2)
{
  if (!s2) s2 = "";
  fprintf (stderr, "%s %s\n", s1, s2);
}

static void stopped() { stopped_f = 1; }
static void control_c() { Exception_normal ("Done"); }
static void shutdown() { Exception_normal ("Done"); }

void CLIUI_takes_over()
{
  char buf[256];
  const int rec_f = Config_iget ("RECORDING");

  sigpipe_handler (shutdown);
  sigterm_handler (shutdown);
  sigint_handler (shutdown);
  
  sprintf (buf, "basename = %s", Config_sget ("BASENAME"));
  CLIUI_msg (buf);
  
  if (!rec_f) {     /* simplified behaviour when only playing */
  	sigint_handler (stopped);
  	task_play();
  	sigint_handler (control_c);
  	Exception_normal ("Done playing.");
  	/* NOT REACHED */
  }
  
  if (Config_iget ("FORCE_ACTION")) {
  	sigint_handler (stopped);
  	task_record();
  	sigint_handler (control_c);
  	Task_write();
  	Exception_normal ("Done recording/saving.");
  	/* NOT REACHED */
  }
  
  do {
  	int c;
  	CLIUI_msg ("Input 'p' to play, 's' to save recorded sample");
  	CLIUI_msg ("   RETURN to record a new sample, Ctrl-C to stop/quit");
  	
  	c = get_key();
  	c = tolower (c);
  	
  	switch (c) {
  	case 'p':
  		sigint_handler (stopped);
  		task_play();
  		sigint_handler (control_c);
  		break;
  	case 's':
  		Task_write();
  		break;
  	case 'r':
  	case '\n':
  	case 0:
  		sigint_handler (stopped);
  		task_record();
  		sigint_handler (control_c);
  		break;
  	default:
  		fprintf (stderr, "Unsupported key: %c\n", c);
  	}
  } while (1);
}

void CLIUI_msg (const char* m)
{
  if (!Config_iget ("INVISIBLE"))
  	fprintf (stdout, "%s\n", m);
}

void CLIUI_time (const char* p)
{
}

void CLIUI_size (long l1, long l2)
{
}

void CLIUI_rbc (int r, int b, int c)
{
}

void CLIUI_close()
{
}

/*----------------------------------------------------------------*/

static int get_key()
{
  char buf[16];
  fgets (buf, 16, stdin);
  return buf[0];
}

void task_play()
{
  int b=1;
  Task_schedule ('P');
  do {
  	sleepms (100);
  	b = Task_timeout();
  }
  while (!stopped_f && b);
  Task_schedule ('S');
  stopped_f = 0;
}

void task_record()
{
  int b=1;
  Task_schedule ('R');
  do {
  	sleepms (100);
  	b = Task_timeout();
  }
  while (!stopped_f && b);
  Task_schedule ('S');
  stopped_f = 0;
}